/*******************************************************************************
  MPLAB Harmony Application Source File
  
  Company:
    Microchip Technology Inc.
  
  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It 
    implements the logic of the application's state machine and it may call 
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013-2014 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/
// DOM-IGNORE-END


// *****************************************************************************
// *****************************************************************************
// Section: Included Files 
// *****************************************************************************
// *****************************************************************************

#include <time.h>
#include <string.h>
#include "app.h"
#include "rtc.h"                        // time()       Implementation for PIC32MX
#include "system/debug/sys_debug.h"

#define HEART_RATE
//#define HEALTH_THERMOMETER

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.
    
    Application strings and buffers are be defined outside this structure.
*/

struct {
    enum {
        APP_STATE_START = 0,
        APP_STATE_IDLE,
    }                           state;
    time_t                      tm;
    USB_HOST_BLUETOOTH_EVENT    btEvent;
}   appData;

// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

bool APP_UsbHostBluetoothEvent(USB_HOST_BLUETOOTH_INDEX index, USB_HOST_BLUETOOTH_EVENT event, void* pData) {
    appData.btEvent = event;
    switch(event){
    case USB_HOST_BLUETOOTH_EVENT_ATTACH:
        PLIB_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_8, 1);
        break;
    case USB_HOST_BLUETOOTH_EVENT_DETACH:
        PLIB_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_8, 0);
        break;
    default:
        SYS_DEBUG_BreakPoint();
        break;
    }
    return(0);
}

//
//  Bluetooth call back routines
//
void bt_rfcommServerDataRecived(unsigned char *inBuff, size_t inLen) {
    static char buff[10];
    static int  pos;
//  int         i;
    static int  i;   // DEBUG

    // Sample code (Extracting of the one line )
    while(inLen){
        for(i = 0; i < inLen; i++) {
            if (pos >= sizeof(buff)) {
                break;
            }
            if ((*inBuff == '\r') || (*inBuff == '\n')) {
                buff[pos++] = '\0';
                inBuff++;
                break;
            } else if (*inBuff >= 0x80) {
                inBuff++;   // skip
            } else {
                buff[pos++] = *inBuff++;
            }
        }
        if (i < inLen) {
            if (strncmp(buff, "off", sizeof(buff)) == 0) {          // LED OFF
                PLIB_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_7, 0);
            } else if (strncmp(buff, "on", sizeof(buff)) == 0) {    // LED ON
                PLIB_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_7, 1);
            }
            pos = 0;
        }
        inLen -= i;
    }
    return;
}

void bt_attDbNotification(unsigned short handle, void *value, size_t valueLen) {

    // Essential action
    bt_att_serverNotificationQueueAdd(handle);

    // User action
    //-------------------------------------------------------------
    switch(handle) {
#if defined(HEART_RATE)
    case 0x1022: // Heart_Rate_Control_Point
        if (*(unsigned char *)value == 1) {
            PLIB_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_7, 1);
        } else {
            PLIB_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_7, 0);
        }
        break;
#elif defined(HEALTH_THERMOMETER)
    case 0x1032: // Measurement_Interval
        if (*(unsigned char *)value == 1) {
            PLIB_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_7, 1);
        } else {
            PLIB_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_7, 0);
        }
        break;
#else   // Sample program
    case 0x1002:
        if (*(unsigned char *)value == 1) {
            PLIB_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_7, 1);
        } else {
            PLIB_PORTS_PinWrite(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_7, 0);
        }
        break;
#endif
    default:
        break;
    }
    //-------------------------------------------------------------
    return;
}

// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************

/* TODO:  Add any necessary local functions.
*/


// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Initialize(void)
{
    appData.state    = APP_STATE_START;
    appData.btEvent  = USB_HOST_BLUETOOTH_EVENT_DETACH;

    USB_HOST_Bluetooth_EventHandlerSet(APP_UsbHostBluetoothEvent);
    USB_HOST_BusEnable(0);
    rtc_open();

    return;
}


/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Tasks(void)
{
    switch(appData.state) {

    case APP_STATE_START:
        if(USB_HOST_BusIsEnabled(0)) {
            appData.state = APP_STATE_IDLE;
        }
        break;

    case APP_STATE_IDLE:
        if (appData.tm != time(NULL)) { // 1sec interval
            appData.tm = time(NULL);

            // Test of RFCOMM
            bt_rfcommServerDataSendRequest((unsigned char *)"Hello world\r\n", 13);

            // Test of ATT(Attribute protocol)
            // Test of ATT
#if defined(HEART_RATE)
            {
                unsigned char buff[2];

                buff[0] = 0x00;             // Flag
                buff[1] = appData.tm % 30 + 60;      // Value(uint8) 60-90[BPM]
                bt_attDbUpdate(0x1002, buff, sizeof(buff));	// Heart Rate Measurement update and send(notify)
            }
#elif defined(HEALTH_THERMOMETER)
            {
                unsigned char buff[5];
                long          temp;

                buff[0] = 0x00;	        // flag
                temp = t % 1000;        // 0.0 - 999.9

                // IEEE 11073 32bit float
                buff[4] = -1;           // exponent
                buff[3] = temp >> 16;
                buff[2] = temp >> 8;
                buff[1] = temp;
                bt_attDbUpdate(0x1002, buff, sizeof(buff));	// Temperature_Measurement update and send(notify)
                bt_attDbUpdate(0x1022, buff, sizeof(buff));	// Intermediate_Temperature update and send(notify)
            }
#else	    // Sample
            {
                bt_attDbUpdate(0x1012, &appData.tm, sizeof(appData.tm));
            }
#endif
        }
        break;

    default:
        break;
    }
    return;
}
 

/*******************************************************************************
 End of File
 */
